#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
implements an image view to show a colored image of a hdf5 dataset.
'''

if __name__ == '__main__':
  #Used to guarantee to use at least Wx2.8
  import wxversion
  wxversion.ensureMinimal('2.8')
import wx
import matplotlib as mpl
if __name__ == '__main__':
  mpl.use('WXAgg')
  #or mpl.use('WX')
  #matplotlib.get_backend()

import os,h5py
import numpy as np
import utilities as ut
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
import pylab as plt #used for the colormaps
try:
  from libDetXR.procMoment import ProcMoment
except ImportError as e:
  print 'ImportError: '+e.message


#from scipy import ndimage as ndi


#or from matplotlib.backends.backend_wx import FigureCanvasWx as FigureCanvas
#The source of the DraggableColorbar is from:
#http://www.ster.kuleuven.be/~pieterd/python/html/plotting/interactive_colorbar.html

#class ShiftedLogNorm(mpl.colors.Normalize):
class ShiftedLogNorm(mpl.colors.LogNorm):
  #copied and modified from LogNorm
    def __call__(self, value, clip=None):
      #print value.shape,self.vmin,self.vmax,self.clip,clip
      if clip is None:
          clip = self.clip 
      ofs0=1-self.vmin
      ofs1=1./(np.log(self.vmax+1-self.vmin))
      result=np.log(value+ofs0)*ofs1
      result = np.ma.masked_less_equal(result, 0, copy=False)
      return result
    def inverse(self, value):
        if not self.scaled():
            raise ValueError("Not invertible until scaled")
        vmin, vmax = self.vmin, self.vmax
        ofs0=1-vmin
        if mpl.cbook.iterable(value):
          val = np.ma.asarray(value)
          return vmin * np.ma.power((vmax/vmin), val)-ofs0
        else:
          return vmin * pow((vmax/vmin), value)-ofs0
    def autoscale_None(self, A):
      if self.vmin is None:
          self.vmin = np.ma.min(A)
      if self.vmax is None:
          self.vmax = np.ma.max(A)
      pass
    def autoscale(self, A):
      pass

class MPLCanvasImg(FigureCanvas):
  def __init__(self,parent,SetStatusCB=None):
    if SetStatusCB:
      self.SetStatusCB=SetStatusCB
    fig = mpl.figure.Figure()
    ax = fig.add_axes([0.075,0.1,0.75,0.85])
    FigureCanvas.__init__(self,parent, -1, fig)
    self.mpl_connect('motion_notify_event', self.OnMotion)
    self.mpl_connect('button_press_event',   self.OnBtnPress) 
    self.mpl_connect('button_release_event', self.OnBtnRelease) 
    self.mpl_connect('scroll_event', self.OnBtnScroll)
    self.mpl_connect('key_press_event',self.OnKeyPress) 
    self.fig=fig
    self.ax=ax
  
  def InitChild(self,data):
    fig=self.fig
    ax=self.ax
    avg=np.average(data); std=np.std(data)
    vmin=np.min(data);vmax=np.max(data)
    vmin=max(vmin,avg-3*std);vmax=min(vmax,avg+3*std)
    
    vmin=1
    norm=ShiftedLogNorm()
    #img = ax.imshow(data,interpolation='nearest',cmap=mpl.cm.jet, norm=ShiftedLogNorm(vmin=vmin, vmax=vmax))

    img = ax.imshow(data,interpolation='nearest',cmap=mpl.cm.jet, vmin=vmin, vmax=vmax)
    colBar=fig.colorbar(img,orientation='vertical',norm=norm)
    colBar.norm=ShiftedLogNorm(vmin=vmin, vmax=vmax)
    img.set_norm(colBar.norm)
    img.cmap._init();bg=img.cmap._lut[0].copy();bg[:-1]/=4
    ax.set_axis_bgcolor(bg)    
    

    self.colBar=colBar
    self.colCycle = sorted([i for i in dir(plt.cm) if hasattr(getattr(plt.cm,i),'N')])
    self.colIndex = self.colCycle.index(colBar.get_cmap().name)
    self.img=img
 
  def OnMotion(self,event):
    #print event,event.x,event.y,event.inaxes,event.xdata,event.ydata
    if event.inaxes==self.ax:
      x=int(round(event.xdata))
      y=int(round(event.ydata))
      try:
        v=self.img.get_array()[y,x]
      except IndexError as e:
        pass
      else:
        #print x,y,v
        self.SetStatusCB(self.Parent,0,(x,y,v))
    elif event.inaxes==self.colBar.ax:
      colBar=self.colBar
      pt=colBar.ax.bbox.get_points()[:,1]
      nrm=colBar.norm
      vmin,vmax,p0,p1,pS = (nrm.vmin,nrm.vmax,pt[0],pt[1],event.y)
      if isinstance(colBar.norm,mpl.colors.LogNorm):#type(colBar.norm)==mpl.colors.LogNorm does not work...
        vS=0
      else:#scale around point
        vS=vmin+(vmax-vmin)/(p1-p0)*(pS-p0)
      self.SetStatusCB(self.Parent,1,vS)
    try:
      vmin,vmax,p0,p1,pS=self.colBarPressed
    except AttributeError:
      return
      #if event.inaxes != self.cbar.ax: return
    colBar=self.colBar
    #print vmin,vmax,p0,p1,pS,type(colBar.norm)
    #print 'x0=%f, xpress=%f, event.xdata=%f, dx=%f, x0+dx=%f'%(x0, xpress, event.xdata, dx, x0+dx)
    
    if isinstance(colBar.norm,mpl.colors.LogNorm):#type(colBar.norm)==mpl.colors.LogNorm does not work...
      if event.button==1:
        #colBar.norm.vmin=.1
        colBar.norm.vmax=vmax*np.exp((pS-event.y)/100)
        #scale= np.exp((event.y-pS)/100)       
    elif event.button==1:#move top,bottom,both
      pD = event.y - pS
      vD=(vmax-vmin)/(p1-p0)*(pS-event.y)
      colBar.norm.vmin = vmin+vD
      colBar.norm.vmax = vmax+vD
    elif event.button==3:#scale around point
      scale= np.exp((pS-event.y)/100)
      vS=vmin+(vmax-vmin)/(p1-p0)*(pS-p0)
      #print scale,vS
      colBar.norm.vmin = vS-scale*(vS-vmin)
      colBar.norm.vmax = vS-scale*(vS-vmax)
    self.img.set_norm(colBar.norm)#force image to redraw
    colBar.patch.figure.canvas.draw()      
      
  def OnBtnPress(self, event):
    """on button press we will see if the mouse is over us and store some data"""
    #print dir(event.guiEvent)
    if event.inaxes == self.colBar.ax:
      #if event.guiEvent.LeftDClick()==True:
      #    print dlg
      pt=self.colBar.ax.bbox.get_points()[:,1]
      nrm=self.colBar.norm
      self.colBarPressed = (nrm.vmin,nrm.vmax,pt[0],pt[1],event.y)
      #self.colBarPressed = event.x, event.y
      #print self.colBarPressed
      #self.OnMouse(event)
    pass

  def OnBtnRelease(self, event):
    """on release we reset the press data"""
    #self.OnMouse(event)
    try: del self.colBarPressed
    except AttributeError: pass
  
  def OnBtnScroll(self, event):
    #self.OnMouse(event)
    colBar=self.colBar
    if event.inaxes==colBar.ax:
      pt=colBar.ax.bbox.get_points()[:,1]
      nrm=colBar.norm
      vmin,vmax,p0,p1,pS = (nrm.vmin,nrm.vmax,pt[0],pt[1],event.y)
      if isinstance(colBar.norm,mpl.colors.LogNorm):#type(colBar.norm)==mpl.colors.LogNorm does not work...
        scale= np.exp((-event.step)/10)
        colBar.norm.vmax=vmax*scale
      else:#scale around point
        scale= np.exp((-event.step)/10)
        vS=vmin+(vmax-vmin)/(p1-p0)*(pS-p0)
        #print scale,vS
        colBar.norm.vmin = vS-scale*(vS-vmin)
        colBar.norm.vmax = vS-scale*(vS-vmax)
      self.img.set_norm(colBar.norm)#force image to redraw
      colBar.patch.figure.canvas.draw()        

  def OnKeyPress(self, event):
    colCycle=self.colCycle
    colBar=self.colBar
    if event.key=='down':
      self.colIndex += 1
    elif event.key=='up':
      self.colIndex -= 1
    self.colIndex%=len(colCycle)
    cmap = colCycle[self.colIndex]
    colBar.set_cmap(cmap)
    colBar.draw_all()
    self.img.set_cmap(cmap)
    self.img.get_axes().set_title(cmap)
    colBar.patch.figure.canvas.draw()

  def OnMouse(self, event):
    for k in dir(event):
      if k[0]!='_':
        print k,getattr(event,k)

class DlgColBarSetupOld(wx.Dialog):
  def __init__(self,parent):
    wx.Dialog.__init__(self,parent,-1,'Colormap Setup')
    colBar=parent.canvas.colBar
    cmap=colBar.cmap
    nrm=colBar.norm
    txtVMin=wx.StaticText(self,-1,'vmin')
    txtVMax=wx.StaticText(self,-1,'vmax')
    self.edVMin=edVMin=wx.TextCtrl(self,-1,'%g'%nrm.vmin)
    self.edVMax=edVMax=wx.TextCtrl(self,-1,'%g'%nrm.vmax)
    sizer=wx.BoxSizer(wx.VERTICAL)
    fgs=wx.FlexGridSizer(3,2,5,5)
    fgs.Add(txtVMin,0,wx.ALIGN_RIGHT)
    fgs.Add(edVMin,0,wx.EXPAND)
    fgs.Add(txtVMax,0,wx.ALIGN_RIGHT)
    fgs.Add(edVMax,0,wx.EXPAND)
    sizer.Add(fgs,0,wx.EXPAND|wx.ALL,5)

    edVMin.SetFocus()

    btns =  self.CreateButtonSizer(wx.OK|wx.CANCEL)
    sizer.Add(btns,0,wx.EXPAND|wx.ALL,5)
    self.Bind(wx.EVT_BUTTON, self.OnBtnOk, id=wx.ID_OK)

    self.SetSizer(sizer)
    sizer.Fit(self)

  def OnBtnOk(self, event):
    event.Skip()#do not consume (use event to close the window and sent return code)
    print 'OnBtnOk'
    parent=self.GetParent()
    canvas=parent.canvas
    colBar=canvas.colBar
    colBar.norm.vmin=float(self.edVMin.Value)
    colBar.norm.vmax=float(self.edVMax.Value)
    canvas.img.set_norm(colBar.norm)
    #colBar.patch.figure.canvas.draw()
    canvas.draw()      


class DlgColBarSetup(wx.Dialog):
  def __init__(self,parent):
    wx.Dialog.__init__(self,parent,-1,'Colormap Setup')
    colBar=parent.canvas.colBar
    cmap=colBar.cmap
    nrm=colBar.norm    
       
    txtVMin=wx.StaticText(self,-1,'vmin')
    txtVMax=wx.StaticText(self,-1,'vmax')
    txtColMap=wx.StaticText(self,-1,'colormap')   
    self.edVMin=edVMin=wx.TextCtrl(self,-1,'%g'%nrm.vmin,style=wx.TE_PROCESS_ENTER)
    self.edVMax=edVMax=wx.TextCtrl(self,-1,'%g'%nrm.vmax,style=wx.TE_PROCESS_ENTER)

    txtTxrFunc=wx.StaticText(self,-1,'function')   
    self.cbtxrFunc=cbtxrFunc=wx.ComboBox(self, -1, choices=('linear','logarithmic'), style=wx.CB_READONLY)    
    cbtxrFunc.SetSelection(0 if nrm.__class__==mpl.colors.Normalize else 1)
    
    #colMapLst=('Accent', 'Blues', 'BrBG', 'BuGn', 'BuPu', 'Dark2', 'GnBu', 'Greens', 'Greys', 'OrRd', 'Oranges', 'PRGn', 'Paired',
    #'Pastel1', 'Pastel2', 'PiYG', 'PuBu', 'PuBuGn', 'PuOr', 'PuRd', 'Purples', 'RdBu', 'RdGy', 'RdPu', 'RdYlBu', 'RdYlGn', 'Reds',
    #'Set1', 'Set2', 'Set3', 'Spectral', 'YlGn', 'YlGnBu', 'YlOrBr', 'YlOrRd', 'afmhot', 'autumn', 'binary', 'bone', 'brg', 'bwr',
    #'cool', 'coolwarm', 'copper', 'cubehelix', 'flag', 'gist_earth', 'gist_gray', 'gist_heat', 'gist_ncar', 'gist_rainbow', 'gist_stern',
    #'gist_yarg', 'gnuplot', 'gnuplot2', 'gray', 'hot', 'hsv', 'jet', 'ocean', 'pink', 'prism', 'rainbow', 'seismic', 'spectral',
    #'spring', 'summer', 'terrain', 'winter')
    
    colMapLst=('hot','spectral','jet','gray','RdYlBu','hsv','gist_stern','gist_ncar','BrBG','RdYlBu','brg','gnuplot2',
               'prism','rainbow',)    
    
    self.cbColMap=cbColMap=wx.ComboBox(self, -1, choices=colMapLst, style=wx.CB_READONLY)
    cbColMap.Value=cmap.name
    
    sizer=wx.BoxSizer(wx.VERTICAL)
    fgs=wx.FlexGridSizer(4,2,5,5)
    fgs.Add(txtVMin,0,wx.ALIGN_RIGHT)
    fgs.Add(edVMin,0,wx.EXPAND)
    fgs.Add(txtVMax,0,wx.ALIGN_RIGHT)
    fgs.Add(edVMax,0,wx.EXPAND)
    fgs.Add(txtTxrFunc,0,wx.ALIGN_RIGHT)
    fgs.Add(cbtxrFunc,0,wx.EXPAND)
    fgs.Add(txtColMap,0,wx.ALIGN_RIGHT)
    fgs.Add(cbColMap,0,wx.EXPAND)
    sizer.Add(fgs,0,wx.EXPAND|wx.ALL,5)

    edVMin.SetFocus()

    btns =  self.CreateButtonSizer(wx.OK|wx.CANCEL)
    btnApply=wx.Button(self, -1, 'Apply')
    btns.Add(btnApply, 0, wx.ALL, 5)
    sizer.Add(btns,0,wx.EXPAND|wx.ALL,5)
    self.Bind(wx.EVT_BUTTON, self.OnModify, id=wx.ID_OK)
    self.Bind(wx.EVT_BUTTON, self.OnModify, btnApply)
    #self.Bind(wx.EVT_TEXT_ENTER, self.OnModify, edVMin)
    #self.Bind(wx.EVT_TEXT_ENTER, self.OnModify, edVMax)
    self.Bind(wx.EVT_TEXT, self.OnModify, edVMin)
    self.Bind(wx.EVT_TEXT, self.OnModify, edVMax)
    self.Bind(wx.EVT_COMBOBOX, self.OnModify, cbtxrFunc)
    self.Bind(wx.EVT_COMBOBOX, self.OnModify, cbColMap)

    self.SetSizer(sizer)
    sizer.Fit(self)

  def OnModify(self, event):
    #print 'OnModify'
    parent=self.GetParent()
    canvas=parent.canvas
    colBar=canvas.colBar
    cmap=colBar.cmap
    nrm=colBar.norm    
    img=canvas.img
    ax=img.get_axes()
    data=img.get_array()

    v=self.cbColMap.Value
    if v!=cmap.name:
      cmap=getattr(mpl.cm,v)
      colBar.set_cmap(cmap)
      colBar.draw_all()
      img.set_cmap(cmap)
      ax.set_title(cmap.name)
      colBar.patch.figure.canvas.draw()

    vmin,vmax=(float(self.edVMin.Value),float(self.edVMax.Value))
    nrm.vmin=vmin; nrm.vmax=vmax
    v=self.cbtxrFunc.GetCurrentSelection()
    func=(mpl.colors.Normalize,ShiftedLogNorm)
    if nrm.__class__!=func[v]:
      if v==0: #linear mapping
        colBar.norm = mpl.colors.Normalize(vmin, vmax)
      elif v==1: #log mapping
        img.cmap._init();bg=img.cmap._lut[0].copy();bg[:-1]/=4
        ax.set_axis_bgcolor(bg)
        vmin=1
        colBar.norm = mpl.colors.LogNorm(vmin,vmax)
    img.set_norm(colBar.norm)
    colBar.patch.figure.canvas.draw()            
    parent.Refresh(False)
    if event.GetId()==wx.ID_OK:
      event.Skip()#do not consume (use event to close the window and sent return code)  
  
class HdfImageFrame(wx.Frame):
  def __init__(self, parent,lbl,hid):
    wx.Frame.__init__(self, parent, title=lbl, size=wx.Size(850, 650))
    imgDir=ut.Path.GetImage()
    icon = wx.Icon(os.path.join(imgDir,'h5pyViewer.ico'), wx.BITMAP_TYPE_ICO)
    self.SetIcon(icon)

    t=type(hid)
    if t==h5py.h5d.DatasetID:
      data=h5py.Dataset(hid)

    canvas =  MPLCanvasImg(self,self.SetStatusCB)

    sizer = wx.BoxSizer(wx.VERTICAL)
    sizer.Add(canvas, 1, wx.LEFT | wx.TOP | wx.GROW)
    self.SetSizer(sizer)

    toolbar=ut.AddToolbar(canvas,sizer)  

    wxAxCtrlLst=[]
    l=len(data.shape)
    idxXY=(l-2,l-1)
    for idx,l in enumerate(data.shape):
      if idx in idxXY:
        continue 
      wxAxCtrl=ut.SliderGroup(self, label='Axis:%d'%idx,range=(0,l-1))
      wxAxCtrl.idx=idx
      wxAxCtrlLst.append(wxAxCtrl)
      sizer.Add(wxAxCtrl.sizer, 0, wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, border=5)
      wxAxCtrl.SetCallback(HdfImageFrame.OnSetView,wxAxCtrl)

    sl=ut.GetSlice(idxXY,data.shape,wxAxCtrlLst)
    
    canvas.InitChild(data[sl])
      
    #self.Fit()   
    self.Centre()
    
    self.BuildMenu()
    self.canvas=canvas
    self.sizer=sizer
    self.toolbar=toolbar
    self.data=data
    self.idxXY=idxXY
    self.wxAxCtrlLst=wxAxCtrlLst
 
  def BuildMenu(self):
    mnBar = wx.MenuBar()

    #-------- Edit Menu --------
    mn = wx.Menu()
    mnItem=mn.Append(wx.ID_ANY, 'Setup Colormap', 'Setup the color mapping ');self.Bind(wx.EVT_MENU, self.OnColmapSetup, mnItem)
    mnItem=mn.Append(wx.ID_ANY, 'Invert X-Axis', kind=wx.ITEM_CHECK);self.Bind(wx.EVT_MENU, self.OnInvertAxis, mnItem)
    self.mnIDxAxis=mnItem.GetId()
    mnItem=mn.Append(wx.ID_ANY, 'Invert Y-Axis', kind=wx.ITEM_CHECK);self.Bind(wx.EVT_MENU, self.OnInvertAxis, mnItem)
    mnItem=mn.Append(wx.ID_ANY, 'Show Moments', 'Show image moments ', kind=wx.ITEM_CHECK);self.Bind(wx.EVT_MENU, self.OnShowMoments, mnItem)
    self.mnItemShowMoment=mnItem
    mnBar.Append(mn, '&Edit')
    mn = wx.Menu()
    mnItem=mn.Append(wx.ID_ANY, 'Help', 'How to use the image viewer');self.Bind(wx.EVT_MENU, self.OnHelp, mnItem)
    mnBar.Append(mn, '&Help')

    self.SetMenuBar(mnBar)
    self.CreateStatusBar()      
        
  def SetIdxXY(self,x,y):
    self.idxXY=(x,y)
 
  @staticmethod
  def SetStatusCB(obj,mode,v):
    if mode==0:
      obj.SetStatusText( "x= %d y=%d val=%g"%v,0)
    elif mode==1:
      obj.SetStatusText( "Colormap Value %d (drag to scale)"%v,0)
    else:
      raise KeyError('wrong mode')

  @staticmethod
  def OnSetView(usrData,value,msg):
    'called when a slice is selected with the slider controls'
    imgFrm=usrData.slider.Parent
    #imgFrm.img.set_array(imgFrm.data[usrData.value,...])
    data=imgFrm.data
    sl=ut.GetSlice(imgFrm.idxXY,data.shape,imgFrm.wxAxCtrlLst)   

    imgFrm.canvas.img.set_array(data[sl])
    if imgFrm.mnItemShowMoment.IsChecked():
      imgFrm.PlotMoments()       
    imgFrm.canvas.draw()
    pass

  def OnShowMoments(self,event):
    if event.IsChecked():
      base='/scratch/detectorData/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne'
      self.procMoment=pm=ProcMoment()
      pm.SetMskMat(os.path.join(base,'analysis/data/pilatus_valid_mask.mat'),False)   
      #pm.SetProcess('python')
      pm.SetProcess('pyFast')
      self.PlotMoments()
      #self.canvas.img.draw()
      data=self.canvas.img.get_array()
      self.canvas.img.set_array(data)
      fig, ax = plt.subplots(2)
      v=data.sum(axis=0); x=np.arange(v.size); x0=x.sum(); m0=v.sum(); m1=(v*x).sum(); m2=(v*x*x).sum()
      ax[0].plot(v);
      m=m1/m0
      s=np.sqrt( (m2-(m1**2/m0))/m0)     
      xx=1/(s*np.sqrt(2*np.pi))*np.exp(-.5*((x-m)/s)**2)
      ax[0].set_title('%g | %g | %g | %g | %g'%(m0,m1,m2,m,s))
      ax[0].hold(True);ax[0].plot(xx*m0)      
      
      v=data.sum(axis=1);
      ax[1].plot(v);
      
    
      plt.show()
      #print pm.resArr[0:3],pm.resArr[1]/pm.resArr[0],pm.resArr[2]/pm.resArr[0]      
    else:
      for o in self.goMoment:
        o.remove()
      del self.goMoment
      del self.procMoment
    self.canvas.draw()             
      
  def PlotMoments(self):
    data=self.canvas.img.get_array()
    pm=self.procMoment

    #data=ndi.median_filter(data, 3)
    data[pm.mask==False]=0
    #data=np.log(data+1)
    #data[100:110,500:510]=1000 #y,x
    #data[650:850,700:850]=0 #y,x
    #pm.Process(np.log(data+1))
    pm.Process(data)
    xbar, ybar, cov=pm.GetIntertialAxis()

    m=pm.resArr
    m00=m[0];m01=m[1];m10=m[2];m11=m[3];m02=m[4];m20=m[5]
    
    xm = m10 / m00
    ym = m01 / m00
    u11 = (m11 - xm * m01) / m00
    #u11[u11<0.]=0. #processing rounding error
    u20 = (m20 - xm * m10) / m00
    u02 = (m02 - ym * m01) / m00
    a=(u20+u02)/2
    b=np.sqrt(4*u11**2+(u20-u02)**2)/2
    l0=a+b
    l1=a-b
    ang=0.5*np.arctan2(2*u11,(u20-u02))/(2*np.pi)*360. #orientation value 0..1
    exc=np.sqrt(1-l1/l0) #eccentricity :circle=0: http://en.wikipedia.org/wiki/Eccentricity_%28mathematics%29

    print 'xb:%g yb:%g cov:%g %g %g %g  ang:%g exc:%g'%((xm, ym)+tuple(cov.ravel())+(ang,exc))
    #fig, ax = plt.subplots()
    #ax.imshow(data,vmax=100,interpolation='nearest')
    #plt.show()
    ax=self.canvas.img.get_axes()
    try:
      for o in self.goMoment:
        o.remove()
    except AttributeError: pass

    self.goMoment=ProcMoment.PlotMoments(ax, xbar, ybar, cov)
    ax.axis('image')

  def OnHelp(self,event):
    msg='''to change the image selection:
use the toolbar at the bottom to pan and zoom the image
use the scrollbars at the bottom (if present) to select an other slice

to change the colorscale:
drag with left mouse button to move the colorbar up and down
drag with right mouse button to zoom in/out the colorbar at a given point
use mouse weel to zoom in/out the colorbar at a given point
double click left mouse button to set maximum and minimun colorbar values
use cursor up and down to use a different colormap'''
    dlg = wx.MessageDialog(self, msg, 'Help', wx.OK|wx.ICON_INFORMATION)
    dlg.ShowModal()
    dlg.Destroy()
  
  def OnColmapSetup(self,event):
    dlg=DlgColBarSetup(self)
    if dlg.ShowModal()==wx.ID_OK:
      pass
    dlg.Destroy()

  def OnInvertAxis(self,event):
    ax=self.canvas.ax
    #event.Checked()
    if self.mnIDxAxis==event.GetId():
      ax.invert_xaxis()
    else:
      ax.invert_yaxis()
    self.canvas.draw() 
    pass
       
if __name__ == '__main__':
  import os,sys,argparse #since python 2.7
  def GetParser(required=True):   
    fnHDF='/scratch/detectorData/e14472_00033.hdf5'
    #lbl='mcs'
    lbl='pilatus_1'
    #lbl='spec'
    elem='/entry/dataScan00033/'+lbl
    exampleCmd='--hdfFile='+fnHDF+' --elem='+elem
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog='Example:\n'+os.path.basename(sys.argv[0])+' '+exampleCmd+'\n ')
    parser.add_argument('--hdfFile', required=required, default=fnHDF, help='the hdf5 to show')
    parser.add_argument('--elem', required=required, default=elem, help='the path to the element in the hdf5 file')   
    return parser
    args = parser.parse_args()
    return args

  class App(wx.App):
    def OnInit(self):
      parser=GetParser()
      #parser=GetParser(False) # debug with exampleCmd
      args = parser.parse_args()
      try:
        self.fid=fid=h5py.h5f.open(args.hdfFile)
      except IOError as e:
        sys.stderr.write('Unable to open File: '+args.hdfFile+'\n')
        parser.print_usage(sys.stderr)
        return True
      try:
        hid = h5py.h5o.open(fid,args.elem)
      except KeyError as e:
        sys.stderr.write('Unable to open Object: '+args.elem+'\n')
        parser.print_usage(sys.stderr)
        return True
      frame = HdfImageFrame(None,args.elem,hid)
      frame.Show()
      self.SetTopWindow(frame)
      return True

    def OnExit(self):
      self.fid.close()

  ut.StopWatch.Start()
  app = App()
  app.MainLoop()



#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
implements an image view to show a colored image of a hdf5 dataset.
'''

if __name__ == '__main__':
  #Used to guarantee to use at least Wx2.8
  import wxversion
  wxversion.ensureMinimal('2.8')
import wx
import matplotlib as mpl
if __name__ == '__main__':
  mpl.use('WXAgg')
  #or mpl.use('WX')
  #matplotlib.get_backend()

import os,h5py
import numpy as np
import utilities as ut
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
import os,h5py
from GLCanvasImg import *
import pyFAI
from hdfImageGL import HdfImageGLFrame
from glumpy.image.texture import Texture
from scipy import ndimage as ndi

def FindCenter(arr):
  m=ndi.median_filter(arr, 5)
  sx=m.sum(1)
  sy=m.sum(0)
  shape=arr.shape
  xx=np.arange(shape[0])
  yy=np.arange(shape[1])    
  x=(xx*sx).sum()/sx.sum()
  y=(yy*sy).sum()/sy.sum()
  #print x,y
  #import pylab as plt #used for the colormaps
  #plt.figure()
  #plt.subplot(211)
  #plt.plot(sx)
  #plt.subplot(212)
  #plt.plot(sy)
  #plt.show(block=False)
  return (x,y)

class MPLCanvasPyFAI1D(FigureCanvas):
  def __init__(self,parent,SetStatusCB=None):
    if SetStatusCB:
      self.SetStatusCB=SetStatusCB
    fig = mpl.figure.Figure()
    ax = fig.add_axes([0.075,0.1,0.75,0.85])
    FigureCanvas.__init__(self,parent, -1, fig)
    #self.mpl_connect('motion_notify_event', self.OnMotion)
    #self.mpl_connect('button_press_event',   self.OnBtnPress) 
    #self.mpl_connect('button_release_event', self.OnBtnRelease) 
    #self.mpl_connect('scroll_event', self.OnBtnScroll)
    #self.mpl_connect('key_press_event',self.OnKeyPress) 
    self.fig=fig
    self.ax=ax
  
  def InitChild(self,data):
    fig=self.fig
    ax=self.ax
    ctrX,ctrY=self.center=FindCenter(data)
    self.ai = pyFAI.AzimuthalIntegrator(1.e3, ctrX, ctrY, 0.0, 0.0, 0.0, 1.e0, 1.e0) 
    #canvas=self.canvas
    self.numPtTh=int(np.average(data.shape)/2.)
    out=self.ai.xrpd(data,self.numPtTh)
    self.hl=ax.plot(*out)
    ax.set_yscale('log')
    #canvas.data=imgPolar
    #print imgPolar.shape
    #out=ai.xrpd(imgData,1000)
    #out=ai.xrpd_OpenCL(imgData,1000)
    #import pylab
    #pylab.plot(*out)
    #pylab.yscale("log")
    #pylab.show()


  
class HdfPyFAI1DFrame(wx.Frame):
  def __init__(self, parent,lbl,hid):
    wx.Frame.__init__(self, parent, title=lbl, size=wx.Size(850, 650))
    imgDir=ut.Path.GetImage()
    icon = wx.Icon(os.path.join(imgDir,'h5pyViewer.ico'), wx.BITMAP_TYPE_ICO)
    self.SetIcon(icon)

    t=type(hid)
    if t==h5py.h5d.DatasetID:
      data=h5py.Dataset(hid)

    canvas =  MPLCanvasPyFAI1D(self,self.SetStatusCB)

    sizer = wx.BoxSizer(wx.VERTICAL)
    sizer.Add(canvas, 1, wx.LEFT | wx.TOP | wx.GROW)
    self.SetSizer(sizer)

    toolbar=ut.AddToolbar(canvas,sizer)  

    wxAxCtrlLst=[]
    l=len(data.shape)
    idxXY=(l-2,l-1)
    for idx,l in enumerate(data.shape):
      if idx in idxXY:
        continue 
      wxAxCtrl=ut.SliderGroup(self, label='Axis:%d'%idx,range=(0,l-1))
      wxAxCtrl.idx=idx
      wxAxCtrlLst.append(wxAxCtrl)
      sizer.Add(wxAxCtrl.sizer, 0, wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, border=5)
      wxAxCtrl.SetCallback(HdfPyFAI1DFrame.OnSetView,wxAxCtrl)

    sl=ut.GetSlice(idxXY,data.shape,wxAxCtrlLst)
    
    canvas.InitChild(data[sl])
      
    #self.Fit()   
    self.Centre()
    
    self.BuildMenu()
    self.canvas=canvas
    self.sizer=sizer
    self.toolbar=toolbar
    self.data=data
    self.idxXY=idxXY
    self.wxAxCtrlLst=wxAxCtrlLst
 
  def BuildMenu(self):
    mnBar = wx.MenuBar()

    #-------- Edit Menu --------
    mn = wx.Menu()
    #mnItem=mn.Append(wx.ID_ANY, 'Setup Colormap', 'Setup the color mapping ');self.Bind(wx.EVT_MENU, self.OnColmapSetup, mnItem)
    #mnItem=mn.Append(wx.ID_ANY, 'Linear Mapping', 'Use a linear values to color mapping ');self.Bind(wx.EVT_MENU, self.OnMapLin, mnItem)
    #mnItem=mn.Append(wx.ID_ANY, 'Log Mapping', 'Use a logarithmic values to color mapping ');self.Bind(wx.EVT_MENU, self.OnMapLog, mnItem)
    #mnItem=mn.Append(wx.ID_ANY, 'Invert X-Axis', kind=wx.ITEM_CHECK);self.Bind(wx.EVT_MENU, self.OnInvertAxis, mnItem)
    #self.mnIDxAxis=mnItem.GetId()
    #mnItem=mn.Append(wx.ID_ANY, 'Invert Y-Axis', kind=wx.ITEM_CHECK);self.Bind(wx.EVT_MENU, self.OnInvertAxis, mnItem)
    mnBar.Append(mn, '&Edit')
    mn = wx.Menu()
    #mnItem=mn.Append(wx.ID_ANY, 'Help', 'How to use the image viewer');self.Bind(wx.EVT_MENU, self.OnHelp, mnItem)
    mnBar.Append(mn, '&Help')

    self.SetMenuBar(mnBar)
    self.CreateStatusBar()      
        
  def SetIdxXY(self,x,y):
    self.idxXY=(x,y)
 
  @staticmethod
  def SetStatusCB(obj,mode,v):
    if mode==0:
      obj.SetStatusText( "x= %d y=%d val=%g"%v,0)
    elif mode==1:
      obj.SetStatusText( "Colormap Value %d (drag to scale)"%v,0)
    else:
      raise KeyError('wrong mode')

  @staticmethod
  def OnSetView(usrData,value,msg):
    'called when a slice is selected with the slider controls'
    imgFrm=usrData.slider.Parent
    #imgFrm.img.set_array(imgFrm.data[usrData.value,...])
    data=imgFrm.data
    sl=ut.GetSlice(imgFrm.idxXY,data.shape,imgFrm.wxAxCtrlLst)

    hl=imgFrm.canvas.hl
    ai=imgFrm.canvas.ai
    numPtTh=imgFrm.canvas.numPtTh
    out=ai.xrpd(data[sl],numPtTh)
    hl[0].set_ydata(out[1])
    imgFrm.canvas.draw()
    pass


###########################################

class HdfPyFAIFrame(HdfImageGLFrame):
  def __init__(self, parent, title, hid):
    HdfImageGLFrame.__init__(self, parent, title, hid)
    #HdfPyFAI1DFrame(self, title, hid)
    canvas=self.canvas
    raw=canvas.data
    ctrX,ctrY=FindCenter(raw)
    self.ai = pyFAI.AzimuthalIntegrator(1.e3, ctrX, ctrY, 0.0, 0.0, 0.0, 1.e0, 1.e0) 
    
    raw
    self.numPtTh=int(np.average(raw.shape)/2.)
    self.numPtCh=360
    
    imgPolar,theta,chi=self.ai.xrpd2(raw,self.numPtTh,self.numPtCh)
    canvas.data=imgPolar
    print imgPolar.shape

  def BuildMenu(self):
    HdfImageGLFrame.BuildMenu(self)
    mnBar=self.GetMenuBar()
    mn=mnBar.GetMenu(0)
    itemLst=mn.GetMenuItems()
    it=itemLst[0]
    it.GetItemLabel()
    mnItem=mn.Append(wx.ID_ANY, 'Setup FAI', 'Setup fast azimutal integration ');self.Bind(wx.EVT_MENU, self.OnFAISetup, mnItem)

  @staticmethod
  def OnSetView(usrData,value,msg):
    'called when a slice is selected with the slider controls'
    frm=usrData.slider.Parent
    ds=frm.dataSet
    canvas=frm.canvas
    glImg=canvas.glImg
    sl=ut.GetSlice(frm.idxXY,ds.shape,frm.wxAxCtrlLst)
    imgPolar,theta,chi=frm.ai.xrpd2(ds[sl],frm.numPtTh,frm.numPtCh)
    canvas.data[:]=imgPolar[:]
    glImg.data[:]=canvas.GetTxrData()
    glImg.update()
    canvas.OnPaint(None)#force to repaint, Refresh and Update do not force !
    #canvas.Refresh(False)
    #canvas.Update()
    pass

  def OnFAISetup(self, event):
    dlg=DlgSetupPyFAI(self)
    if dlg.ShowModal()==wx.ID_OK:
      pass
    dlg.Destroy()


class DlgSetupPyFAI(wx.Dialog):
  def __init__(self,parent):
    wx.Dialog.__init__(self,parent,-1,'pyFAI Setup')
    ai=parent.ai
    #glColBar=parent.glColBar
    #dataRange=parent.dataRange   
    txtCtrX=wx.StaticText(self,-1,'center X')
    txtCtrY=wx.StaticText(self,-1,'center Y')
    txtNumPtTh=wx.StaticText(self,-1,'number of pt in Theta')
    txtNumPtCh=wx.StaticText(self,-1,'number of pt in Chi')  
    txtMethod=wx.StaticText(self,-1,'method')   



    self.edCtrX=edCtrX=wx.TextCtrl(self,-1,'%g'%ai.get_poni1(),style=wx.TE_PROCESS_ENTER)
    self.edCtrY=edCtrY=wx.TextCtrl(self,-1,'%g'%ai.get_poni2(),style=wx.TE_PROCESS_ENTER)
    self.edNumPtTh=edNumPtTh=wx.TextCtrl(self,-1,'%g'%parent.numPtTh,style=wx.TE_PROCESS_ENTER)
    self.edNumPtCh=edNumPtCh=wx.TextCtrl(self,-1,'%g'%parent.numPtCh,style=wx.TE_PROCESS_ENTER)
    self.cbMethod=cbMethod=wx.ComboBox(self, -1, choices=('default','numny'), style=wx.CB_READONLY)
    #cbtxrFunc.SetSelection(parent.txrTrfFunc)
    
    sizer=wx.BoxSizer(wx.VERTICAL)
    fgs=wx.FlexGridSizer(4,2,5,5)
    fgs.Add(txtCtrX,0,wx.ALIGN_RIGHT)
    fgs.Add(edCtrX,0,wx.EXPAND)
    fgs.Add(txtCtrY,0,wx.ALIGN_RIGHT)
    fgs.Add(edCtrY,0,wx.EXPAND)
    fgs.Add(txtNumPtTh,0,wx.ALIGN_RIGHT)
    fgs.Add(edNumPtTh,0,wx.EXPAND)
    fgs.Add(txtNumPtCh,0,wx.ALIGN_RIGHT)
    fgs.Add(edNumPtCh,0,wx.EXPAND)
    fgs.Add(txtMethod,0,wx.ALIGN_RIGHT)
    fgs.Add(cbMethod,0,wx.EXPAND)
    sizer.Add(fgs,0,wx.EXPAND|wx.ALL,5)

    #edVMin.SetFocus()

    btns =  self.CreateButtonSizer(wx.OK|wx.CANCEL)
    btnApply=wx.Button(self, -1, 'Apply')
    btns.Add(btnApply, 0, wx.ALL, 5)
    sizer.Add(btns,0,wx.EXPAND|wx.ALL,5)
    self.Bind(wx.EVT_BUTTON, self.OnModify, id=wx.ID_OK)
    self.Bind(wx.EVT_BUTTON, self.OnModify, btnApply)
    #self.Bind(wx.EVT_TEXT, self.OnModify, edCtrX)
    #self.Bind(wx.EVT_TEXT, self.OnModify, edCtrY)
    #self.Bind(wx.EVT_TEXT, self.OnModify, edNumSector)
    self.Bind(wx.EVT_COMBOBOX, self.OnModify, cbMethod)
    self.SetSizer(sizer)
    sizer.Fit(self)

  def OnModify(self, event):
    print 'OnModify'
    frm=self.GetParent() 
    ds=frm.dataSet
    canvas=frm.canvas
    glImg=canvas.glImg
    ai=frm.ai
    ai.set_poni1(float(self.edCtrX.Value))
    ai.set_poni2(float(self.edCtrY.Value))
    frm.numPtTh=int(self.edNumPtTh.Value)
    frm.numPtCh=int(self.edNumPtCh.Value)
    sl=ut.GetSlice(frm.idxXY,ds.shape,frm.wxAxCtrlLst)
    imgPolar,theta,chi=frm.ai.xrpd2(ds[sl],frm.numPtTh,frm.numPtCh)
    if canvas.data.shape==imgPolar.shape:
      canvas.data[:]=imgPolar[:]
      glImg.data[:]=canvas.GetTxrData()
    else:
      canvas.data=imgPolar;
      glImg._data=canvas.GetTxrData()
      glImg._texture=Texture(glImg._data)
      #self.glImg=glImg=glumpy.image.Image(txrData, colormap=colMap,vmin=txrRng[0], vmax=txrRng[1])
      print canvas.data.shape,glImg.data.shape
    glImg.update()
    canvas.OnPaint(None)#force to repaint, Refresh and Update do not force !
    frm.Refresh(False)
    if event.GetId()==wx.ID_OK:
      event.Skip()#do not consume (use event to close the window and sent return code)
  
  
if __name__ == '__main__':
  import os,sys,argparse #since python 2.7
  def GetParser(required=True):
    fnHDF='/scratch/detectorData/e14472_00033.hdf5'
    #lbl='mcs'
    lbl='pilatus_1'
    #lbl='spec'
    elem='/entry/data/'+lbl
    exampleCmd='--hdfFile='+fnHDF+' --elem='+elem
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog='Example:\n'+os.path.basename(sys.argv[0])+' '+exampleCmd+'\n ')
    parser.add_argument('--hdfFile', required=required, default=fnHDF, help='the hdf5 to show')
    parser.add_argument('--elem', required=required, default=elem, help='the path to the element in the hdf5 file')
    return parser
    args = parser.parse_args()
    return args

  class App(wx.App):
    def OnInit(self):
      parser=GetParser()
      #parser=GetParser(False) # debug with exampleCmd
      args = parser.parse_args()
      try:
        self.fid=fid=h5py.h5f.open(args.hdfFile)
      except IOError as e:
        sys.stderr.write('Unable to open File: '+args.hdfFile+'\n')
        parser.print_usage(sys.stderr)
        return True
      try:
        hid = h5py.h5o.open(fid,args.elem)
      except KeyError as e:
        sys.stderr.write('Unable to open Object: '+args.elem+'\n')
        parser.print_usage(sys.stderr)
        return True
      #frame = HdfPyFAIFrame(None,args.elem,hid)
      frame = HdfPyFAI1DFrame(None,args.elem,hid)
      frame.Show()
      self.SetTopWindow(frame)
      return True

    def OnExit(self):
      self.fid.close()

  ut.StopWatch.Start()
  app = App()
  app.MainLoop()



#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
implements an image view to show a colored image of a hdf5 dataset.
'''

if __name__ == '__main__':
    #Used to guarantee to use at least Wx2.8
  import wxversion
  wxversion.ensureMinimal('2.8')
import wx
import os,h5py
import numpy as np
import utilities as ut
from GLCanvasImg import *
  
class HdfImageGLFrame(wx.Frame):
  def __init__(self, parent, title, hid):
        # Forcing a specific style on the window.
        #   Should this include styles passed?
    style = wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE
    wx.Frame.__init__(self, parent, title=title, size=wx.Size(850, 650), style=style)
    imgDir=ut.Path.GetImage()
    icon = wx.Icon(os.path.join(imgDir,'h5pyViewer.ico'), wx.BITMAP_TYPE_ICO)
    self.SetIcon(icon)
    canvas=GLCanvasImg(self,self.SetStatusCB)

    t = type(hid)
    if t == h5py.h5d.DatasetID:
      ds = h5py.Dataset(hid)
      self.dataSet=ds

    sizer = wx.BoxSizer(wx.VERTICAL)
    sizer.Add(canvas, 1, wx.LEFT | wx.TOP | wx.GROW)
    self.SetSizer(sizer)

    wxAxCtrlLst=[]
    l=len(ds.shape)
    idxXY=(l-2,l-1)
    for idx,l in enumerate(ds.shape):
      if idx in idxXY:
        continue 
      wxAxCtrl=ut.SliderGroup(self, label='Axis:%d'%idx,range=(0,l-1))
      wxAxCtrl.idx=idx
      wxAxCtrlLst.append(wxAxCtrl)
      sizer.Add(wxAxCtrl.sizer, 0, wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, border=5)
      wxAxCtrl.SetCallback(self.OnSetView,wxAxCtrl)

    sl=ut.GetSlice(idxXY,ds.shape,wxAxCtrlLst)
    

    canvas.data=ds[sl]
      
    #self.Fit()   
    self.Centre()
    
    self.canvas=canvas
    self.sizer=sizer
    self.idxXY=idxXY
    self.wxAxCtrlLst=wxAxCtrlLst
    self.BuildMenu()

  def BuildMenu(self):
    mnBar = wx.MenuBar()

    #-------- Edit Menu --------
    mn = wx.Menu()
    mnItem=mn.Append(wx.ID_ANY, 'Setup Colormap', 'Setup the color mapping ');self.Bind(wx.EVT_MENU, self.canvas.OnColmapSetup, mnItem)
    #mnItem=mn.Append(wx.ID_ANY, 'Linear Mapping', 'Use a linear values to color mapping ');self.Bind(wx.EVT_MENU, self.OnMapLin, mnItem)
    #mnItem=mn.Append(wx.ID_ANY, 'Log Mapping', 'Use a logarithmic values to color mapping ');self.Bind(wx.EVT_MENU, self.OnMapLog, mnItem)
    #mnItem=mn.Append(wx.ID_ANY, 'Invert X-Axis', kind=wx.ITEM_CHECK);self.Bind(wx.EVT_MENU, self.OnInvertAxis, mnItem)
    #self.mnIDxAxis=mnItem.GetId()
    #mnItem=mn.Append(wx.ID_ANY, 'Invert Y-Axis', kind=wx.ITEM_CHECK);self.Bind(wx.EVT_MENU, self.OnInvertAxis, mnItem)
    mnBar.Append(mn, '&Edit')
    mn = wx.Menu()
    mnItem=mn.Append(wx.ID_ANY, 'Help', 'How to use the image viewer');self.Bind(wx.EVT_MENU, self.canvas.OnHelp, mnItem)
    mnBar.Append(mn, '&Help')

    self.SetMenuBar(mnBar)
    self.CreateStatusBar()      

  @staticmethod
  def SetStatusCB(obj,mode,v):
    if mode==0:
      obj.SetStatusText( "Pos:(%d,%d) Value:%g"%v,0)

  @staticmethod
  def OnSetView(usrData,value,msg):
    'called when a slice is selected with the slider controls'
    frm=usrData.slider.Parent
    ds=frm.dataSet
    canvas=frm.canvas
    glImg=canvas.glImg
    sl=ut.GetSlice(frm.idxXY,ds.shape,frm.wxAxCtrlLst)
    canvas.data[:]=ds[sl][:]
    glImg.data[:]=canvas.GetTxrData()
    glImg.update()
    canvas.OnPaint(None)#force to repaint, Refresh and Update do not force !
    #canvas.Refresh(False)
    #canvas.Update()
    pass
    
if __name__ == '__main__':
  import os,sys,argparse #since python 2.7
  def GetParser(required=True):
    fnHDF='/scratch/detectorData/e14472_00033.hdf5'
    #lbl='mcs'
    lbl='pilatus_1'
    #lbl='spec'
    elem='/entry/data/'+lbl
    exampleCmd='--hdfFile='+fnHDF+' --elem='+elem
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog='Example:\n'+os.path.basename(sys.argv[0])+' '+exampleCmd+'\n ')
    parser.add_argument('--hdfFile', required=required, default=fnHDF, help='the hdf5 to show')
    parser.add_argument('--elem', required=required, default=elem, help='the path to the element in the hdf5 file')
    return parser
    args = parser.parse_args()
    return args

  class App(wx.App):
    def OnInit(self):
      parser=GetParser()
      #parser=GetParser(False) # debug with exampleCmd
      args = parser.parse_args()
      try:
        self.fid=fid=h5py.h5f.open(args.hdfFile)
      except IOError as e:
        sys.stderr.write('Unable to open File: '+args.hdfFile+'\n')
        parser.print_usage(sys.stderr)
        return True
      try:
        hid = h5py.h5o.open(fid,args.elem)
      except KeyError as e:
        sys.stderr.write('Unable to open Object: '+args.elem+'\n')
        parser.print_usage(sys.stderr)
        return True
      frame = HdfImageGLFrame(None,args.elem,hid)
      frame.Show()
      self.SetTopWindow(frame)
      return True

    def OnExit(self):
      self.fid.close()

  ut.StopWatch.Start()
  app = App()
  app.MainLoop()



#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
hdf5 viewer to dispay images, tables, attributes and trees of a hdf5 file.
'''
import os,sys
import wx,h5py
import wx.py
from hdfTree import *
from hdfGrid import *
from hdfAttrib import *
from hdfImage import  *
try:
  from hdfImageGL import  *
except ImportError as e:
  print 'ImportError: '+e.message
try:
  from FrmPyFAI import  *
except ImportError as e:
  print 'ImportError: '+e.message
try:
  from FrmProcRoiStat import ProcRoiStatFrame
except ImportError as e:
  print 'ImportError: '+e.message
  
import utilities as ut


class AboutFrame(wx.Frame):
  def __init__(self,parent):
    wx.Frame.__init__(self,parent,-1,'About h5pyViewer',size=(300,330))
    imgDir=ut.Path.GetImage()
    icon = wx.Icon(os.path.join(imgDir,'h5pyViewer.ico'), wx.BITMAP_TYPE_ICO)
    self.SetIcon(icon)
    self.Centre()
    panel=wx.Panel(self,-1)
    import pkg_resources

    v=pkg_resources.get_distribution("h5pyViewer")
    s='Version:'+str(v)+'\n(c) www.psi.ch\n Author: Thierry Zamofing\n thierry.zamofing@psi.ch'

    st0=wx.StaticText(panel,-1,s,(30,10))
    bmp = wx.StaticBitmap(panel,-1,wx.Bitmap(os.path.join(imgDir,'splash1.png'), wx.BITMAP_TYPE_ANY ), (30,st0.Position[1]+st0.Size[1]+10))

class HdfTreePopupMenu(wx.Menu):
  def __init__(self, wxObjSrc):
    wx.Menu.__init__(self)
    self.wxObjSrc=wxObjSrc
    self.AddMenu(self.OnShowAttrib,"Show Attributes")
    self.AddMenu(self.OnShowData,"Show Data")
    self.AddMenu(self.OnShowImage,"Show Image")
    self.AddMenu(self.OnShowImageGL,"Show Image OpenGL")
    self.AddMenu(self.OnShowImgFAI1D,"Show Azimutal Integral Image 1D")
    self.AddMenu(self.OnShowImgFAI2D,"Show Azimutal Integral Image 2D")
    self.AddMenu(self.OnShowRoiStat,"Show Roi Statistics")   
    self.AddMenu(self.OnShell,"Python Shell")
    self.AddMenu(self.OnPrintProperties,"Print Properties")
    self.AddMenu(self.OnItem2,"Item Two")
    self.AddMenu(self.OnItem3,"Item Three")

  def AddMenu(self,func,lbl):
    item = wx.MenuItem(self, -1, lbl)
    self.AppendItem(item);
    self.Bind(wx.EVT_MENU, func, item)
    return item
    
  def OnShowAttrib(self, event):
    wxTree,wxNode=self.wxObjSrc
    lbl=wxTree.GetItemText(wxNode)
    hid=wxTree.GetPyData(wxNode)
    if type(hid)==h5py.h5f.FileID:
      hid=h5py.h5o.open(hid,'/')     
    frame=HdfAttribFrame(wxTree,lbl,hid)
    frame.Show(True)
    
  def OnShowData(self, event):
    wxTree,wxNode=self.wxObjSrc
    lbl=wxTree.GetItemText(wxNode)
    hid=wxTree.GetPyData(wxNode)
    frame=HdfGridFrame(wxTree,lbl,hid)
    frame.Show(True)  

  def OnShowImage(self, event):
    wxTree,wxNode=self.wxObjSrc
    lbl=wxTree.GetItemText(wxNode)
    hid=wxTree.GetPyData(wxNode)
    frame=HdfImageFrame(wxTree,lbl,hid)
    frame.Show(True)  

  def OnShowImageGL(self, event):
    wxTree,wxNode=self.wxObjSrc
    lbl=wxTree.GetItemText(wxNode)
    hid=wxTree.GetPyData(wxNode)
    frame=HdfImageGLFrame(wxTree,lbl,hid)
    frame.Show(True)     

  def OnShowImgFAI1D(self, event):
    wxTree,wxNode=self.wxObjSrc
    lbl=wxTree.GetItemText(wxNode)
    hid=wxTree.GetPyData(wxNode)
    frame=HdfPyFAI1DFrame(wxTree,lbl,hid)
    frame.Show(True)     
    
  def OnShowImgFAI2D(self, event):
    wxTree,wxNode=self.wxObjSrc
    lbl=wxTree.GetItemText(wxNode)
    hid=wxTree.GetPyData(wxNode)
    frame=HdfPyFAIFrame(wxTree,lbl,hid)
    frame.Show(True)     

  def OnShowRoiStat(self, event):
    wxTree,wxNode=self.wxObjSrc
    lbl=wxTree.GetItemText(wxNode)
    hid=wxTree.GetPyData(wxNode)
    fnMatRoi='/scratch/detectorData/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne/analysis/data/pilatus_integration_mask.mat'    
    frame=ProcRoiStatFrame(wxTree,lbl,hid,fnMatRoi)
    frame.Show(True)     
    
  def OnShell(self, event):
    wxTree,wxNode=self.wxObjSrc
    frame = wx.Frame(wxTree, -1, "wxPyShell",size=wx.Size(800, 500))
    imgDir=ut.Path.GetImage()
    icon = wx.Icon(os.path.join(imgDir,'h5pyViewer.ico'), wx.BITMAP_TYPE_ICO)
    frame.SetIcon(icon)
    frame.Centre()
    wnd=app.GetTopWindow() 
    loc={'app'  :app,
         'fid'  :app.GetTopWindow().fid,
         'lbl'  :wxTree.GetItemText(wxNode),
         'hid'  :wxTree.GetPyData(wxNode),
         'h5py' : h5py
         }
    introText='''Shell to the HDF5 objects
app: application object
fid: hdf5 file object
lbl: label of selected hdf5 object
hid: selected hdf5 object

#Examples:
import h5py
ds=h5py.Dataset(hid)
ds[1,:,:]

#using user defined modules
import userSample as us;reload(us);us.test1(hid)

'''
    shell=wx.py.shell.Shell(frame, introText=introText,locals=loc)
    frame.Show(True)
    #if loc is None, all variables are visible. the context is global
    #shell.push('wnd=app.GetTopWindow()')
    #for cmd in [
    #  'wnd=app.GetTopWindow();wxTree=wnd.wxTree',
    #  'wxNode=wnd.wxTree.GetSelection()',
    #  'print wnd.fid',
    #  'lbl=wxTree.GetItemText(wxNode)',
    #  'hid=wxTree.GetPyData(wxNode)']:
    #  shell.run(cmd, prompt=False)

  def OnPrintProperties(self, event):
    wxTree,wxNode=self.wxObjSrc
    lbl=wxTree.GetItemText(wxNode)
    hid=wxTree.GetPyData(wxNode)
    print HdfViewerFrame.GetPropertyStr(wxTree,wxNode)
    
  def OnItem2(self, event):
    print 'OnItem2'
    pass

  def OnItem3(self, event):
    print 'OnItem3'
    pass

class HdfViewerFrame(wx.Frame):

  def OpenFile(self,fnHDF):
    try:
      self.fid=h5py.h5f.open(fnHDF)
    except IOError as e:
      sys.stderr.write('Unable to open File: '+fnHDF+'\n')
    else: 
      self.wxTree.ShowHirarchy(self.fid)

  def CloseFile(self):
    #http://docs.wxwidgets.org/2.8/wx_windowdeletionoverview.html#windowdeletionoverview
    print 'CloseFile'
    try:
      self.fid.close()
      del self.fid
    except AttributeError as e:
      pass

  def __init__(self, parent, title):
    wx.Frame.__init__(self, parent, title=title, size=wx.Size(650, 350))
    imgDir=ut.Path.GetImage()
    icon = wx.Icon(os.path.join(imgDir,'h5pyViewer.ico'), wx.BITMAP_TYPE_ICO)
    self.SetIcon(icon)
    wxSplt = wx.SplitterWindow(self, -1)
    wxTree = HdfTreeCtrl(wxSplt, 1, wx.DefaultPosition, (-1,-1),  wx.TR_HAS_BUTTONS)
    wxTree.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnSelChanged, id=1)
    wxTree.Bind(wx.EVT_TREE_ITEM_MENU, self.OnMenu, id=1)
    #wx.EVT_TREE_ITEM_MENU(id, func)
    wxTxt = wx.StaticText(wxSplt, -1, '',(10,10) )#, style=wx.ALIGN_CENTRE)

    wxSplt.SplitVertically(wxTree, wxTxt)

    #wxLstCtrl=HdfAttrListCtrl(wxSplt)
    #wxSplt.SplitVertically(wxTree, wxLstCtrl)
    self.BuildMenu()
    
    self.Centre()

    self.wxTree=wxTree
    self.display=wxTxt
  def __del__(self):
    self.CloseFile()
  
  def OnOpen(self, event):
    dlg = wx.FileDialog(self, "Choose a file", os.getcwd(), '','HDF5 files (*.hdf5)|*.hdf5|all (*.*)|*.*', wx.OPEN|wx.FD_CHANGE_DIR)
    if dlg.ShowModal() == wx.ID_OK:
      path = dlg.GetPath()
      #mypath = os.path.basename(path)
      #self.SetStatusText("You selected: %s" % mypath)
      self.CloseFile()
      self.OpenFile(path)
      print 'OnOpen',path
    dlg.Destroy()       
  
  def OnCloseWindow(self, event):
    print 'OnCloseWindow'
    self.Destroy()
    
  def OnAbout(self,event):
    frame=AboutFrame(self)
    frame.Show()
    
  def BuildMenu(self):
    #http://wiki.wxpython.org/AnotherTutorial#wx.MenuBar
    mnBar = wx.MenuBar()

    #-------- File Menu --------
    mn = wx.Menu()
    mnItem=mn.Append(wx.ID_OPEN, '&Open', 'Open a new document');self.Bind(wx.EVT_MENU, self.OnOpen, mnItem)
    #mnSub = wx.Menu()
    #mnItem=mnSub.Append(wx.ID_ANY, 'SubMenuEntry', 'My SubMenuEntry')
    #mn.AppendMenu(wx.ID_ANY, 'SubMenu', mnSub)   
    mn.AppendSeparator()
    mnItem=mn.Append(wx.ID_EXIT, '&Quit', 'Quit the Application');self.Bind(wx.EVT_MENU, self.OnCloseWindow, mnItem)
    mnBar.Append(mn, '&File')
    
    self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

    #-------- Edit Menu --------
    #mn = wx.Menu()
    #mnBar.Append(mn, '&Edit')

    #-------- Help Menu --------
    mn = wx.Menu()
    #mnItem=mn.Append(wx.ID_HELP,'Help','Application Help')
    mnItem=mn.Append(wx.ID_ABOUT,'About','Application About');self.Bind(wx.EVT_MENU, self.OnAbout, mnItem)
    mnBar.Append(mn, '&Help')
          
    #mn.AppendSeparator()
    #mnItem = wx.MenuItem(mn, 105, '&Quit\tCtrl+Q', 'Quit the Application')
    #mnItem.SetBitmap(wx.Image('stock_exit-16.png', wx.BITMAP_TYPE_PNG).ConvertToBitmap())
    #mn.AppendItem(mnItem)   
    self.SetMenuBar(mnBar)
    self.CreateStatusBar()   

  @staticmethod
  def GetPath(wxTree,wxNode):
    if wxTree.GetRootItem()==wxNode:
      hid=wxTree.GetPyData(wxNode)
      return hid.name
    wxNodeParent=wxTree.GetItemParent(wxNode)
    if wxTree.GetRootItem()==wxNodeParent:
      return wxTree.GetItemText(wxNode)
    else:
      return HdfViewerFrame.GetPath(wxTree,wxNodeParent)+'/'+wxTree.GetItemText(wxNode)

  @staticmethod
  def GetPropertyStr(wxTree,wxNode):
    
    path=HdfViewerFrame.GetPath(wxTree,wxNode)
    
    hidStr=wxTree.GetItemText(wxNode)
    hid=wxTree.GetPyData(wxNode)
    #o=wxTree.GetItemData(wxNode)
    #print o.Data,wxTree.GetPyData(wxNode)
    #if type(gid)==h5py.h5g.GroupID:
    txt=path+'\n'
    t=type(hid)
    if t==tuple:
      if hid[0]==None:
        txt+='missing external link:\n  '+hid[1]+'\n  '+hid[2]
        return txt
      else:
        txt+='external link:\n  '+hid[1]+'\n  '+hid[2]
        hid=hid[0]
        t=type(hid)
    if t==h5py.h5f.FileID:
      txt+=type(hid).__name__+':%d\n'%hid.id     
      hid=h5py.h5o.open(hid,'/')
      t=type(hid)
    objInf=h5py.h5o.get_info(hid)
    #print t,hid.id,objInf.fileno, objInf.rc, objInf.type, objInf.addr, objInf.hdr       
    txt+=type(hid).__name__+':%d\n'%hid.id     
    txt+='addr:%d fileno:%d refCnt:%d\n'%(objInf.addr,objInf.fileno, objInf.rc)      
    try:
      wxNodeParent=wxTree.GetItemParent(wxNode)
      txtParent=wxTree.GetItemText(wxNode)
      dataParent=wxTree.GetPyData(wxNode)
      gid=wxTree.GetPyData(wxNodeParent)
      softLnk=gid.get_linkval(hidStr)
    except BaseException as e:
      pass
    else:
      txt+='Soft Link:'+softLnk+'\n'
    try: numAttr=h5py.h5a.get_num_attrs(hid)
    except ValueError as e:
      pass
    else:
      txt+='Attributes:%d\n'%numAttr
    if t==h5py.h5g.GroupID:
      pass
    elif t==h5py.h5d.DatasetID:
      txt+='shape: '+str(hid.shape)+'\n'
      tt=hid.get_type()
      ttt=type(tt)
      if ttt==h5py.h5t.TypeCompoundID:
        txt+='type: Compound\n'
      elif ttt==h5py.h5t.TypeStringID:
        sz=tt.get_size()
        txt+='type: String (length %d)\n'%sz
      else:
        txt+='type: '+str(tt.dtype)+'\n'

    
      pl=hid.get_create_plist()
      txFcn=(
       ('chunk',h5py.h5p.PropDCID.get_chunk),
       ('fill time',   h5py.h5p.PropDCID.get_fill_time),
       ('alloc_time',  h5py.h5p.PropDCID.get_alloc_time),
       #('class',       h5py.h5p.PropDCID.get_class),
       #('fill_value',  h5py.h5p.PropDCID.get_fill_value),
       #('filter',      h5py.h5p.PropDCID.get_filter),
       #('filter_by_id',h5py.h5p.PropDCID.get_filter_by_id),
       ('layout',      h5py.h5p.PropDCID.get_layout),
       ('nfilters',    h5py.h5p.PropDCID.get_nfilters),
       #('obj_track_times', h5py.h5p.PropDCID.get_obj_track_times),
       )
      for tx,func in txFcn:
        try: v=func(pl)
        except ValueError as e: pass
        else:txt+=tx+':'+str(v)+'\n'
      if hid.shape==(1,):
        data=h5py.Dataset(hid)
        txt+='value:'+str(data[0])+'\n'
    return txt
    
  def OnSelChanged(self, event):
      wxNode =  event.GetItem()
      txt=HdfViewerFrame.GetPropertyStr(self.wxTree,wxNode)            
      self.display.SetLabel(txt)

  def OnMenu(self, event):
      wxNode =  event.GetItem()
      self.PopupMenu(HdfTreePopupMenu((self.wxTree,wxNode)), event.GetPoint())    

if __name__ == '__main__':
  def GetArgs():   
    import sys,argparse #since python 2.7
    fnHDF='/scratch/detectorData/e14472_00033.hdf5'

    exampleCmd='--hdfFile='+fnHDF    
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog='Example:\n'+os.path.basename(sys.argv[0])+' '+exampleCmd+'\n ')
    parser.add_argument('hdfFile',   nargs='?', default=fnHDF, help='the hdf5 to show')
    
    args = parser.parse_args()
    return args

  class MyApp(wx.App):
    def OnInit(self):
      args=GetArgs()
      frame = HdfViewerFrame(None, 'h5pyViewer')
      frame.OpenFile(args.hdfFile)
      frame.Show(True)
      self.SetTopWindow(frame)
      return True
    
#------------------ Main Code ----------------------------------    
  app = MyApp()
  app.MainLoop()



#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
implements an image view to show a colored image of a hdf5 dataset.
'''

import wx
#import os,h5py
import numpy as np

import glumpy
from glumpy.graphics import VertexBuffer
import wx.glcanvas
from OpenGL.GL import *
#from scipy import ndimage as ndi

def MplAddColormap(m,lut):
  if type(lut)==dict:
    try:
      lstR=lut['red']
      lstG=lut['green']
      lstB=lut['blue']
      kR,vR,dummy=zip(*lstR)
      kG,vG,dummy=zip(*lstG)
      kB,vB,dummy=zip(*lstB)
    except TypeError as e:
      print 'failed to add '+m+' (probably some lambda function)'
      #print lut
      return
    kLst=set()
    kLst.update(kR)
    kLst.update(kG)
    kLst.update(kB)
    kLst=sorted(kLst)
    vRGB=zip(np.interp(kLst, kR, vR),np.interp(kLst, kG, vG),np.interp(kLst, kB, vB))
    lut2=zip(kLst,vRGB)
  else:
    if type(lut[0][1])==tuple:
      lut2=lut
    else:
      kLst=np.linspace(0., 1., num=len(lut))
      lut2=zip(kLst,lut)
  
  #cmap = Colormap('gray',(0., (0.,0.,0.,1.)),(1., (1.,1.,1.,1.)))
  cm2=glumpy.colormap.Colormap(m,*tuple(lut2))
  setattr(glumpy.colormap,m,cm2)  
  
def MplAddAllColormaps(colMapNameLst=None):
  try:
    import matplotlib.cm as cm
  except ImportError as e:
    print 'ImportError: '+e.message
  if not colMapNameLst:
    colMapNameLst=[m for m in cm.datad if not m.endswith("_r")]
  for m in colMapNameLst:
    lut= cm.datad[m]
    MplAddColormap(m,lut)
  pass 

class GLCanvasImg(wx.glcanvas.GLCanvas):
  """A simple class for using OpenGL with wxPython."""
  def __init__(self,parent,SetStatusCB=None):
    if SetStatusCB:
      self.SetStatusCB=SetStatusCB
    self.GLinitialized = False
    attribList = (wx.glcanvas.WX_GL_RGBA,  # RGBA
                  wx.glcanvas.WX_GL_DOUBLEBUFFER,  # Double Buffered
                  wx.glcanvas.WX_GL_DEPTH_SIZE, 24)  # 24 bit

    # Create the canvas
    #self.canvas = wx.glcanvas.GLCanvas(parent, attribList=attribList)
    wx.glcanvas.GLCanvas.__init__(self, parent, attribList=attribList)

    # Set the event handlers.
    self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
    self.Bind(wx.EVT_SIZE, self.OnSize)
    self.Bind(wx.EVT_PAINT, self.OnPaint)
    self.Bind(wx.EVT_MOUSEWHEEL, self.OnMouseWeel)
    self.Bind(wx.EVT_MOTION, self.OnMouseEvent)
    self.Bind(wx.EVT_LEFT_DOWN, self.OnMouseEvent)
    self.Bind(wx.EVT_LEFT_UP, self.OnMouseEvent)
    
  def OnMouseEvent(self, event):
    try:
      data=self.data
    except AttributeError:
      return
    if event.ButtonDown(0):
      self.mouseStart=(np.array(event.GetPosition()),self.imgCoord.copy())
      print 'drag Start'
    elif event.ButtonUp(0):
      print 'drag End'
      del self.mouseStart
    else:
      try:
        (pStart,icStart)=self.mouseStart
      except AttributeError as e:
        pSz = np.array(self.GetClientSize(),np.float32)
        pMouse  = np.array(event.GetPosition(),np.float32)
        ic=self.imgCoord
        pOfs=(ic[0::4]+[.5,.5])*pSz
        tPos=(pMouse-pOfs)/(pSz-2*pOfs)#position on the image 0..1
        #print tPos 
        if (tPos<0).any() or (tPos>1).any(): return
        tPos=tPos*(ic[3::4]-ic[1::4])+ic[1::4]
        tPos[0]*=data.shape[1]
        tPos[1]*=data.shape[0]
        v=tuple(tPos.astype(np.int32))
        #v=tuple(reversed(v))
        v+=(data[tuple(reversed(v))],)
        self.SetStatusCB(self.Parent,0,v)

        #vS=event.GetPosition()[0]
        pass
      else: 
        #prefix:
        #p Pixel, t Texture, v vertex
        pSz = np.array(self.GetClientSize(),np.float32)
        pMouse  = np.array(event.GetPosition(),np.float32)
        ic=self.imgCoord

        pOfs=(ic[0::4]+[.5,.5])*pSz
        tOfs=(pMouse-pStart)/(pSz-2*pOfs)#position on the image 0..1
        tOfs=tOfs*(icStart[3::4]-icStart[1::4])
        
        if icStart[1]-tOfs[0]<0:
          tOfs[0]=icStart[1]
        if icStart[5]-tOfs[1]<0:
          tOfs[1]=icStart[5]
        if icStart[3]-tOfs[0]>1:
          tOfs[0]=icStart[3]-1
        if icStart[7]-tOfs[1]>1:
          tOfs[1]=icStart[7]-1
          
        #print icStart[1::4],icStart[3::4],tOfs

        ic[1::4]=icStart[1::4]-tOfs
        ic[3::4]=icStart[3::4]-tOfs
            
        self.SetZoom()
        self.Refresh(False)
    #event.Skip()
    pass

  def OnMouseWeel(self, event):
    #prefix:
    #p Pixel, t Texture, v vertex
    pSz = np.array(self.GetClientSize(),np.float32)
    pMouse  = np.array(event.GetPosition(),np.float32)
    ic=self.imgCoord    
    n=event.GetWheelRotation()
    pOfs=(ic[0::4]+[.5,.5])*pSz
    tPos=(pMouse-pOfs)/(pSz-2*pOfs)#position on the image 0..1
    if n>0:
      z=0.3
    else:
      z=-0.3
    tMin=tPos*z
    tMax=tMin+(1.0-z)
    tMin=tMin*(ic[3::4]-ic[1::4])+ic[1::4]
    tMax=tMax*(ic[3::4]-ic[1::4])+ic[1::4]
    tMin[tMin<0]=0
    tMax[tMax>1]=1
    ic[1::4]=tMin
    ic[3::4]=tMax
    #print tPos,pSz,pMouse,n,ic
    self.SetZoom()
    self.Refresh(False)

    pass

  def OnEraseBackground(self, event):
    """Process the erase background event."""
    #print 'OnEraseBackground'
    pass # Do nothing, to avoid flashing on MSWin

  def OnSize(self, event):
    """Process the resize event."""
    #print 'OnSize'
    if self.GetContext():
            # Make sure the frame is shown before calling SetCurrent.
      self.Show()
      self.SetCurrent()

      size = self.GetClientSize()
      self.Reshape(size.width, size.height)
      self.Refresh(False)

    event.Skip()

  def OnPaint(self, event):
    """Process the drawing event."""
    #print 'OnPaint'
    self.SetCurrent()

    # This is a 'perfect' time to initialize OpenGL ... only if we need to
    if not self.GLinitialized:
      self.InitGL()
      self.GLinitialized = True
      size = self.GetClientSize()
      self.Reshape(size.width, size.height)
      self.Refresh(False)

    glClear(GL_COLOR_BUFFER_BIT)
    ic=self.imgCoord
    try: glImg=self.glImg
    except AttributeError: pass
    else:  self.glImg.draw(ic[0],ic[4],0,ic[2]-ic[0],ic[6]-ic[4])
    self.glColBar.draw(-.5,-.5,0,1.,.02)       
    # Drawing an example triangle in the middle of the screen
    #glBegin(GL_TRIANGLES)
    #glColor(1, 0, 0)
    #glVertex(-.25, -.25)
    #glVertex(.25, -.25)
    #glVertex(0, .25)
    #glEnd()
    self.SwapBuffers()
    if event!=None:
      event.Skip()

  def OnHelp(self,event):
    msg='''to change the image selection:
drag with left mouse button to move the image
use mouse wheel to zoom in/out the image at a given point
'''
    dlg = wx.MessageDialog(self, msg, 'Help', wx.OK|wx.ICON_INFORMATION)
    dlg.ShowModal()
    dlg.Destroy()

  def OnColmapSetup(self,event):
    dlg=DlgColBarSetup(self)
    if dlg.ShowModal()==wx.ID_OK:
      pass
    dlg.Destroy()
    
  def SetZoom(self):
    ic=self.imgCoord
    xmin,xmax,ymin,ymax=ic[1::2];
    vert=self.glImg._vertices
    n = vert.shape[0]
    u,v = np.mgrid[0:n,0:n]
    u=u*(xmax-xmin)/float(n-1)+xmin
    v=v*(ymax-ymin)/float(n-1)+ymin
    #u/=2;u+=.3
    vert['tex_coord']['u'] = u
    vert['tex_coord']['v'] = v


  def UpdateImg(self):
    try:
      txrData=self.GetTxrData()
    except AttributeError as e:
      return
    try:
      glImg=self.glImg
    except AttributeError as e:
      colMap=self.glColBar._filter.colormap
      txrRng=self.GetTxrRange()
      self.glImg=glImg=glumpy.image.Image(txrData, colormap=colMap,vmin=txrRng[0], vmax=txrRng[1])
    else:
      if glImg.data.shape==self.data.shape:
        glImg.data[:]=txrData[:]
      else:
        self.glImg=glImg=glumpy.image.Image(txrData, colormap=colMap,vmin=glImg._vmin, vmax=glImg._vmax)   
    glImg.update()

  def GetTxrRange(self):
    txrTrfFunc=self.txrTrfFunc
    dataRange=self.dataRange
    if txrTrfFunc==0:
      return dataRange
    elif txrTrfFunc==1:
      return (0,np.log(1+dataRange[1]-dataRange[0]))
  
  def AutoRange(self,txrTrfFunc):
    data=self.data
    self.txrTrfFunc=txrTrfFunc
    if txrTrfFunc==0:
      avg=np.average(data); std=np.std(data)
      vmin=data.min();vmax=data.max()
      vmin=max(vmin,avg-3*std);vmax=min(vmax,avg+3*std)      
    elif txrTrfFunc==1:
      avg=np.average(data); std=np.std(data)     
      vmin=data.min();vmax=data.max()
      vmin=max(vmin,avg-3*std);vmax=min(vmax,avg+3*std)                 
    self.dataRange=(vmin,vmax)

  def GetTxrData(self):
    data=self.data
    try:
      txrTrfFunc=self.txrTrfFunc
    except AttributeError as e:
      self.AutoRange(1)     
      txrTrfFunc=self.txrTrfFunc

    if txrTrfFunc==0:
      if data.dtype==np.float32:
        txrData=data
      else:
        txrData=data[...].astype(np.float32)
    elif txrTrfFunc==1:
      ofs=1.-self.dataRange[0]
      txrData=np.log(data[...].astype(np.float32)+ofs)
    return txrData
  
  def InitGL(self):
    """Initialize OpenGL for use in the window."""
    #print 'InitGL'
    glClearColor(.2, .2, .2, 1)
    colMap=glumpy.colormap.Hot
    txrColBar=np.linspace(0.,1., 256).astype(np.float32)
    self.glColBar=glumpy.image.Image(txrColBar, colormap=colMap,vmin=0, vmax=1)
   
    self.UpdateImg()
    self.imgCoord=np.array([-.49,0,.49,1,-.49,0,.49,1])#xmin,xmax,umin,umax,ymin,ymax,vmin,vmax    
    pass

  def Reshape(self, width, height):
    """Reshape the OpenGL viewport based on the dimensions of the window."""
    #print 'Reshape'
    glViewport(0, 0, width, height)

    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    glOrtho(-0.5, 0.5, -0.5, 0.5, -1, 1)

    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

class DlgColBarSetup(wx.Dialog):
  def __init__(self,parent):
    wx.Dialog.__init__(self,parent,-1,'Colormap Setup')
    #glImg=parent.glImg
    #glColBar=parent.glColBar
    dataRange=parent.dataRange   
    txtVMin=wx.StaticText(self,-1,'vmin')
    txtVMax=wx.StaticText(self,-1,'vmax')
    txtColMap=wx.StaticText(self,-1,'colormap')   
    self.edVMin=edVMin=wx.TextCtrl(self,-1,'%g'%dataRange[0],style=wx.TE_PROCESS_ENTER)
    self.edVMax=edVMax=wx.TextCtrl(self,-1,'%g'%dataRange[1],style=wx.TE_PROCESS_ENTER)

    txtTxrFunc=wx.StaticText(self,-1,'function')   
    self.cbtxrFunc=cbtxrFunc=wx.ComboBox(self, -1, choices=('linear','logarithmic'), style=wx.CB_READONLY)
    cbtxrFunc.SetSelection(parent.txrTrfFunc)
    
    colMapLst=[]
    #adding all existing colormaps
    #MplAddAllColormaps()
    #for (k,v) in glumpy.colormap.__dict__.iteritems():
    #  if isinstance(v,glumpy.colormap.Colormap):
    #    colMapLst.append(k)
        
    #adding best existing colormaps of glumpy and mpl
    for k in ('Hot','spectral','jet','Grey','RdYlBu','hsv','gist_stern','gist_rainbow','IceAndFire','gist_ncar'):
      try:
        v=glumpy.colormap.__dict__[k]
      except KeyError as e:
        try:
          import matplotlib.cm as cm
          lut= cm.datad[k]
          MplAddColormap(k,lut)
          v=glumpy.colormap.__dict__[k]
        except ImportError as e:
          print e.message
          print "don't have colormap "+k
          continue
      if isinstance(v,glumpy.colormap.Colormap):
        colMapLst.append(k)

    self.cbColMap=cbColMap=wx.ComboBox(self, -1, choices=colMapLst, style=wx.CB_READONLY)
    cbColMap.Value=parent.glImg._filter.colormap.name
    
    sizer=wx.BoxSizer(wx.VERTICAL)
    fgs=wx.FlexGridSizer(4,2,5,5)
    fgs.Add(txtVMin,0,wx.ALIGN_RIGHT)
    fgs.Add(edVMin,0,wx.EXPAND)
    fgs.Add(txtVMax,0,wx.ALIGN_RIGHT)
    fgs.Add(edVMax,0,wx.EXPAND)
    fgs.Add(txtTxrFunc,0,wx.ALIGN_RIGHT)
    fgs.Add(cbtxrFunc,0,wx.EXPAND)
    fgs.Add(txtColMap,0,wx.ALIGN_RIGHT)
    fgs.Add(cbColMap,0,wx.EXPAND)
    sizer.Add(fgs,0,wx.EXPAND|wx.ALL,5)

    edVMin.SetFocus()

    btns =  self.CreateButtonSizer(wx.OK|wx.CANCEL)
    btnApply=wx.Button(self, -1, 'Apply')
    btns.Add(btnApply, 0, wx.ALL, 5)
    sizer.Add(btns,0,wx.EXPAND|wx.ALL,5)
    self.Bind(wx.EVT_BUTTON, self.OnModify, id=wx.ID_OK)
    self.Bind(wx.EVT_BUTTON, self.OnModify, btnApply)
    #self.Bind(wx.EVT_TEXT_ENTER, self.OnModify, edVMin)
    #self.Bind(wx.EVT_TEXT_ENTER, self.OnModify, edVMax)
    self.Bind(wx.EVT_TEXT, self.OnModify, edVMin)
    self.Bind(wx.EVT_TEXT, self.OnModify, edVMax)
    self.Bind(wx.EVT_COMBOBOX, self.OnModify, cbtxrFunc)
    self.Bind(wx.EVT_COMBOBOX, self.OnModify, cbColMap)

    self.SetSizer(sizer)
    sizer.Fit(self)

  def OnModify(self, event):
    #print 'OnModify'
    parent=self.GetParent()
    glImg=parent.glImg
    glColBar=parent.glColBar

    v=self.cbColMap.Value
    if v!=glImg._filter.colormap.name:
      cmap=getattr(glumpy.colormap,v)
      glImg._filter.colormap=cmap
      glImg._filter.build()
      glColBar._filter.colormap=cmap
      glColBar._filter.build()
      glColBar.update()   
    
    parent.dataRange=(float(self.edVMin.Value),float(self.edVMax.Value))
    v=self.cbtxrFunc.GetCurrentSelection()
    if v==1 or v!=parent.txrTrfFunc:
      parent.txrTrfFunc=v  
      txrRange=parent.GetTxrRange()   
      glImg._vmin, glImg._vmax = txrRange
      parent.UpdateImg()
    else:
      txrRange=parent.GetTxrRange()   
      glImg._vmin, glImg._vmax = txrRange
      glImg.update()
    parent.Refresh(False)
    if event.GetId()==wx.ID_OK:
      event.Skip()#do not consume (use event to close the window and sent return code)
  


#! /usr/bin/env python

#FOR DISTUTILS READ:
#http://docs.python.org/2/distutils/index.html

import distutils.core, distutils.dist
import distutils.command.sdist, distutils.command.build_py,distutils.command.install_lib
import re

import sys,os,platform,subprocess
#python setup.py --help-commands
#python setup.py --help

#python setup.py --name --version --fullname --author --author-email  --maintainer  --maintainer-email --contact --contact-email  --url  --license  --description --long-description --platforms --classifiers --keywords --provides --requires --obsoletes

#python setup.py build
#python setup.py build

#python setup.py install --prefix=~/ 
#python setup.py bdist_egg

#python setup.py install --root tmp
#python setup.py bdist --formats=wininst
#python setup.py bdist_wininst
#python setup.py bdist

#Command Line : --help-commands
#Standard commands:
#  build            build everything needed to install
#  build_py         "build" pure Python modules (copy to build directory)
#  build_ext        build C/C++ extensions (compile/link to build directory)
#  build_clib       build C/C++ libraries used by Python extensions
#  build_scripts    "build" scripts (copy and fixup #! line)
#  clean            clean up temporary files from 'build' command
#  install          install everything from build directory
#  install_lib      install all Python modules (extensions and pure Python)
#  install_headers  install C/C++ header files
#  install_scripts  install scripts (Python or otherwise)
#  install_data     install data files
#  sdist            create a source distribution (tarball, zip file, etc.)
#  register         register the distribution with the Python package index
#  bdist            create a built (binary) distribution
#  bdist_dumb       create a "dumb" built distribution
#  bdist_rpm        create an RPM distribution
#  bdist_wininst    create an executable installer for MS Windows
#  upload           upload binary package to PyPI
#  check            perform some checks on the package

def getVersion():
  #for dirname, dirnames, filenames in os.walk('.'):
  #  for subdirname in dirnames:
  #    print os.path.join(dirname, subdirname)
  #  for filename in filenames:
  #    print os.path.join(dirname, filename)

  fn='./PKG-INFO'
  if os.access(fn, os.R_OK):
    sys.stdout.write('getVersion() -> Parsing '+fn)
    fo=open(fn,'r')
    for ln in fo.readlines():
      if ln.startswith('Version:'):
        ver=re.match('Version:\s*(\S*)', ln).group(1)
      elif ln.startswith('Summary:'):
        #print ln
        gitcmt=re.search('\(git:(.*)\)', ln).group(1)
    fo.close()
  else:
    argv=sys.argv
    p = subprocess.Popen('git rev-list HEAD', shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    #p = subprocess.Popen('git log --pretty=%h', shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    retval = p.wait()
    res=p.stdout.readlines()
    ver=len(res)
    ver='0.0.0.'+str(ver)
    gitcmt=res[0][:7]
    sys.stdout.write('getVersion() -> using git command')
  print ':'+ver+':'+gitcmt
  return (ver,gitcmt)

def runSetup(**kv):
  ver=getVersion()

  args={#'cmdclass'    :{'sdist': MySDIST, 'build_py': MyBUILD_PY, 'install_lib':MyINSTALL_LIB},
        'name'        :'h5pyViewer',
        'version'     : ver[0],
        'description' :'(git:'+ver[1]+') HDF5-File-Viewer',
        'author'      :'Thierry Zamofing',
        'author_email':'thierry.zamofing@psi.ch',
        'maintainer'  :'Thierry Zamofing',
        'maintainer_email':'thierry.zamofing@psi.ch',
        'url'         :'www.psi.ch',
        'license'     :'(c) 2013 PSI',
        'long_description':open('README.rst', 'r').read(),
        'platforms'   : ['Linux','Windows'],
        #'py_modules'  :['libDetXR', 'cbfParser'],
        'packages'    :['h5pyViewer'],
        #'package_dir' :{'h5pyViewer':'.'},
        'package_data':{'h5pyViewer': ['images/*.png','images/*.ico']},
        #'requires' requires: h5py==2.0.1 libDetXR==0.0.0.6 numpy==1.7.1 matplotlib==1.2.0 
        'requires' : ['ctypes','h5py','numpy','matplotlib']
      }
  if kv:
    args.update(kv)
  distutils.core.setup(**args)

  print 'done'
  pass

def main():
  argv=sys.argv
  if 'debug' in argv:
    script_args=['--name', '--version', '--fullname', '--author', '--author-email', '--maintainer', '--maintainer-email', '--contact', '--contact-email', '--url', '--license', '--description', '--long-description', '--platforms', '--classifiers', '--keywords', '--provides', '--requires', '--obsoletes']
    runSetup(script_args=script_args)
  else:
    runSetup()
  pass

if __name__ == '__main__':
    main()

